@@ -71,6 +71,12 @@ class DerivedColumn(object):
         return describe_column(self.name, self.type)
 
 
+class AggregationColumn(object):
+
+    def __init__(self):
+        pass
+
+
 class DerivedTableColumn(object):
 
     """Not so much a derived column, but a column on a
@@ -111,7 +117,3 @@ class DerivedTableColumn(object):
 
 class JoinColumn(DerivedTableColumn):
     pass
-
-
-class JoinColumn(DerivedTableColumn):
-    pass
